#!/usr/local/bin/ruby
#
require "test/unit" 
lib_dir = File.expand_path( File.dirname( __FILE__) + '/../lib' )
$:.unshift lib_dir
require '30secondrule/mop'
  

#-- =====================================================
#++
# Example class for testing.
class Foo
  def initialize( name='foo' )  
    @name = name
  end


 # An example method 
  def testme( s=nil, t=nil )
   s ? "#{@name} has #{s.to_s}#{t.to_s}" : "I have no args!"
  end

  def moo( s=nil )
    s ? s.to_s  : "MOO!"
  end
 
end

#-- =====================================================
#++
# Another example class
class Bar
  def testme( s=nil )
   s ?  "Bar has #{s.to_s}"  : "Bar.testme"
  end
end

# Yes, one more example class
class Gullible
  def initialize(  name )
    @name = name 
  end

  def herbal_cialas
    "Sure, I respond to herbal cialas!"
  end

  def bank_account_details
     "#{@name}: #{@name.intern.to_i}"
  end
end



#-- =====================================================
#++
class TC_MOO < Test::Unit::TestCase
  
  #-- =====================================================
  #++
  def setup
    $foo = Foo.new( "Test!" )
    $bar = Bar.new
  end
 
  
  #-- =====================================================
  #++
  def teardown
    $foo = nil
    $bar = nil
    ObjectSpace.garbage_collect
  end

  #-- =====================================================
  #++
  def test_01_no_args1
    res =  :testme.>> $foo 
    assert( res[0] == "I have no args!", "test_01_no_args1 got #{res.inspect}" ) 
  end
   
  #-- =====================================================
  #++
  def test_02_with_args
    res =  :testme[ "My String" ].>>( $foo )
    assert( res[0] == "Test! has My String", "Got #{res[0]}" ) 
    res =  :testme[ ["My String", "Other string"] ].>>( $foo )
    assert( res[0] == "Test! has My StringOther string", "Got #{res[0]}" ) 
  end
   
     
  #-- =====================================================
  #++
  def test_04_call_selected
    pk = lambda { |o| 
        res  = ( o.class == Foo ? true : false  )
        res
      }
      
      res =  :testme[ "My String" ].>>( *pk  )
      assert( res[0] == "Test! has My String", "test_04_call_selected Got #{res[0]}" ) 
    end
     
  #-- =====================================================
  #++
  def test_05_call_selected_by_proc_args

    pk = lambda { |o| 
        res  = o.respond_to?(  "moo" ) ? true : false 
        puts(  o.inspect ) if res
        res
      }
      
      res =  :testme[ "My String" ].>>( pk  )
      assert( res[0] == "Test! has My String", "test_05_call_selected_by_proc Got {res[0]}" ) 
      res = nil

    end


   def test_06_call_selected_by_proc_no_args

    pk = lambda { |o| 
      res  = o.respond_to?(  "moo" ) ? true : false 
      puts(  o.inspect ) if res
      res
    }
    res =  :testme.>>( pk  )

    assert_equal "I have no args!", res[0] 
    assert( res[0] == "I have no args!", "test_06_call_selected_by_proc_no_args got #{res[0]}" ) 
    res = nil 
  end
  
  #-- =====================================================
  #++
  def test_07_mulit_objects
    res =  :testme[ "My String" ].>>( $foo, $bar )
    assert( res[0] == "Test! has My String", "Got '#{res[0]}'" ) 
    res = nil
  end


  def test_gullible
     jim = Gullible.new( 'Jim' )    
     greg = Gullible.new( 'Greg' )

     poor_souls_accounts = :bank_account_details.>>( lambda { |o| o.respond_to?(  :herbal_cialas ) })
     assert_equal 2, poor_souls_accounts.size
     assert_equal ["Greg: 17221", "Jim: 17229"], poor_souls_accounts
     
  end
end

